{*******************************************************************************
* Unit      : KeyDB                                                            *
********************************************************************************
* Purpose   : Handles the interface to the key file which holds the keys       *
********************************************************************************
* Copyright : This module is copyright TSM Inc. (Ian Sparkes) 1998.            *
*             All rights reserved. Liability limited to replacement of         *
*             this original source code in the case of loss or damage because  *
*             the use or misuse of this software.                              *
********************************************************************************
* Version   : 28.03.98  - 1.0 Original module                                  *
********************************************************************************
* Switches  : none                                                             *
*******************************************************************************}
unit KeyDB;

interface

uses
     Classes;

{*******************************************************************************
* Type      : TKeyRecord                                                       *
********************************************************************************
* Purpose   : Groups the elements which make up a key                          *
*             Name - the name of the owner of the secret/key pair              *
*             PubKey - the public key                                          *
*             SecKey - indicator if we know the secret                         *
*******************************************************************************}
type TKeyRecord = record
     Name: string;
     PubKey: string;
     SecKey: Boolean;
end; {TKeyRecord}

function  GetKey(const KeyName: string; var KeyRecord: TKeyRecord): Boolean;
procedure GetKeyList(ListOfKeys: TStrings; Secrets: Boolean);
procedure SaveKey(KeyRecord: TKeyRecord);
function  ParseKey(const StringToParse: string): TKeyRecord;

implementation

uses
     SysUtils;

{*******************************************************************************
* Procedure : SaveKey                                                          *
********************************************************************************
* Purpose   : Appends the current key record at the end of the key file        *
********************************************************************************
* Paramters : KeyRecord - the key record to be stored                          *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure SaveKey(KeyRecord: TKeyRecord);
var
     Outfile: TextFile;
     KeyString: string;
begin
     AssignFile(Outfile, 'PubKey.txt');

     // check if the key file is there
     if FileExists('PubKey.txt') then
     begin
          // if yes, then append to it
          Reset(Outfile);
          Append(OutFile);
          FileMode := 1;
     end
     else
     begin
          // if not, then create it
          Rewrite(Outfile);
     end; {if}

     // dump the contents of the key record as a string
     KeyString := KeyRecord.Name + ';' + KeyRecord.PubKey + ';';

     // if we have the secret then store this too
     if KeyRecord.SecKey then
     begin
          KeyString := KeyString + 'True';
     end
     else
     begin
          KeyString := KeyString + 'False';
     end; {if}

     Writeln(Outfile, KeyString);
     CloseFile(Outfile);
end; {SaveKey}

{*******************************************************************************
* Procedure : GetKey                                                           *
********************************************************************************
* Purpose   : Searches for the key name in the key file                        *
********************************************************************************
* Paramters : KeyName - the name of the key owner                              *
********************************************************************************
* Returns   : Result - true if the key was found, otherwise false              *
*             KeyRecord - the key record if found, otherwise undefined         *
*******************************************************************************}
function GetKey(const KeyName: string; var KeyRecord: TKeyRecord): Boolean;
var
     Infile: TextFile;
     Keys: TKeyRecord;
     KeyString: string;
begin
     // look for the key file
     if not FileExists('PubKey.txt') then
     begin
          // if not found then exit with key not found
          Result := False;
          Exit;
     end; {if}

     AssignFile(Infile, 'PubKey.txt');
     Reset(Infile);

     while not EOF(Infile) do
     begin
          // read each of the key records in turn
          Readln(Infile, KeyString);

          // break the key down into its components
          Keys := ParseKey(KeyString);

          if Trim(KeyName) = Trim(Keys.Name) then
          begin
               // read the information out of the file.
               // If we have found the key, then retrieve it and exit
               KeyRecord := Keys;
               Result := True;
               Exit;
          end; {if}
     end; {while}
     CloseFile(InFile);

     // otherwise we have not found the key
     Result := False;
end; {GetKey}

{*******************************************************************************
* Procedure : GetKeyList                                                       *
********************************************************************************
* Purpose   : Retieves all the key names from the file                         *
********************************************************************************
* Paramters : ListOfKeys - the string list where we should store the names     *
*             Secrets - retrieve only secret keys if true, otherwise all keys  *
********************************************************************************
* Returns   : ListOfKeys - the filled list of key names                        *
*******************************************************************************}
procedure GetKeyList(ListOfKeys: TStrings; Secrets: Boolean);
var
     Infile: TextFile;
     Keys: TKeyRecord;
     KeyString: string;
begin
     // clear the list
     ListOfKeys.Clear;

     // look for the key file
     if not FileExists('PubKey.txt') then
     begin
          // if not found then exit
          Exit;
     end; {if}

     AssignFile(Infile, 'PubKey.txt');
     Reset(Infile);

     while not EOF(Infile) do
     begin
          // read each of the keys in turn
          Readln(Infile, KeyString);

          // break the key down into its components
          Keys := ParseKey(KeyString);

          // if we are only retrieveing secret keys, then be selective
          if Secrets then
          begin
               if Keys.SecKey then ListOfKeys.Add(Trim(Keys.Name));
          end
          else
          begin
               ListOfKeys.Add(Trim(Keys.Name));
          end; {if}
     end; {while}

     CloseFile(InFile);
end;

{*******************************************************************************
* Procedure : ParseKey                                                         *
********************************************************************************
* Purpose   : Breaks a key string down into the structured record              *
********************************************************************************
* Paramters : StringToParse - the string to be broken down                     *
********************************************************************************
* Returns   : Result - the string broken down into its components              *
*******************************************************************************}
function ParseKey(const StringToParse: string): TKeyRecord;
var
     KeyString: string;
     SepPos: integer;
begin
     // copy the input string, as we need to manipulate this
     KeyString := StringToParse;

     // find the position of the first separator
     SepPos := Pos(';',KeyString);

     // copy the first part of the separator, and remove it from the input string
     Result.Name := Copy(KeyString,0,SepPos-1);
     KeyString := Copy(KeyString,SepPos+1,Length(KeyString)-SepPos);

     // copy the first part of the separator, and remove it from the input string
     SepPos := Pos(';',KeyString);
     Result.PubKey := Copy(KeyString,0,SepPos-1);
     KeyString := Copy(KeyString,SepPos+1,Length(KeyString)-SepPos);

     Result.SecKey := (KeyString = 'True');
end; {ParseKey}

{*******************************************************************************
* Procedure : Trim                                                             *
********************************************************************************
* Purpose   : Removes leading and trailing spaces from the string              *
********************************************************************************
* Paramters : InputString - the string to be trimmed                           *
********************************************************************************
* Returns   : Result - the string trimmed of leading and trailing spaces       *
*******************************************************************************}
function Trim(InputString: string): string;
var
     BeginPos: integer;
     EndPos: integer;
begin
     BeginPos := 0;
     while InputString[BeginPos] <> ' ' do Inc(BeginPos);
     EndPos := Length(InputString);
     while InputString[EndPos] <> ' ' do Dec(EndPos);
     Result := Copy(InputString,BeginPos,EndPos-BeginPos);
end; {Trim}

end.
